
; Port C pin descriptions
; SCL                   bit = 3
; SDA                   bit = 4
;
; 10MHz crystal is being used, thus each instruction cycle = 400 ns
;
;*******************RAM register definitions**********************
;    cblock  0x000
;        datai                   ; Data input byte buffer
;        datao                   ; Data output byte buffer
;        bytecount               ; Counter for byte loops
;        pollcnt                 ; Counter for polling loops
;        loops                   ; Delay loop counter
;        loops2                  ; Delay loop counter
;    endc
;*******************Macro definitions*****************************
WRITE_ADDR  equ     b'10100000' ; Control byte for write operations
READ_ADDR   equ     b'10100001' ; Control byte for read operations
NUMBYTES    equ     .16         ; # of bytes for page/seq. operations
;*******************Include file**********************************
        ;include "p18f452.inc"       ; Include file for the PIC18F452
;    errorlevel  -302            ; Suppress message 302 from list file
;*******************Initialization subroutine*********************
;           for I2C Master mode, with a 100 kHz clock.
;*****************************************************************
  cblock  0x40
datai                   ;Data input byte buffer
datao                   ;Data output byte buffer
second                  ;
minute                  ;
hours                   ;
year_day                ;    
weekday_month           ;     
year
month
day
weekday
  endc
  
    ;org     0x600
Init8583 macro
    Clrf    SSPSTAT             ; Disable SMBus inputs
    Bsf     SSPSTAT,SMP         ; Disable slew rate control
    Movlw   0x18                ; Load 0x18 into WREG ? - 0x28
    Movwf   SSPADD              ; Setup 100 kHz I2C clock
    Movlw   b'00101000'
    Movwf   SSPCON1             ; Enable SSP, select I2C Master mode, SSPEN=1
    Clrf    SSPCON2             ; Clear control bits
    Bsf     TRISC,3             ;RC0,1,6,7=out, RC3-5=in
    Bsf     TRISC,4             ;  
    Clrf    PORTC               ; PORTC  
    Bcf     PIR1,SSPIF          ; Clear SSP interrupt flag
    Bcf     PIR2,BCLIF          ; Clear Bit Collision flag
    endm

;*******************Start bit subroutine**************************

BSTART
    bcf     PIR1,SSPIF          ; Clear SSP interrupt flag
    bsf     SSPCON2,SEN         ; Generate Start condition
bstart_wait
    btfss   PIR1,SSPIF          ; Check if operation completed
    bra     bstart_wait         ; If not, keep checking
    return

;*******************Restart bit subroutine**************************
BRESTART
    bcf     PIR1,SSPIF          ; Clear SSP interrupt flag
    bsf     SSPCON2,RSEN        ; Generate Restart condition
brestart_wait
    btfss   PIR1,SSPIF          ; Check if operation completed
    bra     brestart_wait       ; If not, keep checking
    return

;*******************Stop bit subroutine***************************

BSTOP
    bcf     PIR1,SSPIF          ; Clear SSP interrupt flag
    bsf     SSPCON2,PEN         ; Generate Stop condition
bstop_wait
    btfss   PIR1,SSPIF          ; Check if operation completed
    bra     bstop_wait          ; If not, keep checking
    return

;*******************Data transmit subroutine**********************
TX_BYTE
    bcf     PIR1,SSPIF          ; Clear SSP interrupt flag
    movf    datao,W             ; Copy datao to WREG
    movwf   SSPBUF              ; Write byte out to device
tx_wait
    btfss   PIR1,SSPIF          ; Check if operation completed
    bra     tx_wait             ; If not, keep checking
ackfailed  
    btfsc   SSPCON2,ACKSTAT     ; Check if ACK bit was received
    bra     ackfailed           ; This executes if no ACK received    

    return

;*******************Data receive subroutine***********************
RX_BYTE
    bsf     SSPCON2,ACKDT       ; Select to send NO ACK bit
    bcf     PIR1,SSPIF          ; Clear SSP interrupt flag
    bsf     SSPCON2,RCEN        ; Initiate reception of byte
rx_wait
    btfss   PIR1,SSPIF          ; Check if operation completed
    bra     rx_wait             ; If not, keep checking
    movf    SSPBUF,W            ; Copy byte to WREG
    movwf   datai               ; Copy WREG to datai
    bcf     PIR1,SSPIF          ; Clear SSP interrupt flag
    bsf     SSPCON2,ACKEN       ; Generate ACK/NO ACK bit    
rx_wait2
    btfss   PIR1,SSPIF          ; Check if operation completed
    bra     rx_wait2            ; If not, keep checking

    return

;*******************Byte ********************
Cloc_Read  macro
    Call Read_Cloc
    endm
    
Data_Read  macro
    Call  Read_Data
    endm
        
Read_Cloc:
;+++++++++++++  +++++++++++++++++++        
    call   BSTART              ; Generate Start condition
    movlw   .160            ; Load control byte for write
    movwf   datao               ; Copy to datao for output
    call   TX_BYTE             ; Send control byte to device
    movlw   3                ; Load 0x5A for word address
    movwf   datao               ; Copy to datao for output
    call   TX_BYTE
    call   BRESTART             ; Generate Start condition  
    movlw   .161                ; Load 0x00 for word address
    movwf   datao               ; Copy to datao for output
    call   TX_BYTE             ; Send word address to device
    call   RX_BYTE             ; Read data byte from device
    Movff  datai,minute    
    call   BSTOP               ; Generate Stop conditio
;+++++++++++++  +++++++++++++++++++       
    call   BSTART              ; Generate Start condition
    movlw   .160            ; Load control byte for write
    movwf   datao               ; Copy to datao for output
    call   TX_BYTE             ; Send control byte to device
    movlw   4                ; Load 0x5A for word address
    movwf   datao               ; Copy to datao for output
    call   TX_BYTE
    call   BRESTART             ; Generate Start condition  
    movlw   .161                ; Load 0x00 for word address
    movwf   datao               ; Copy to datao for output
    call   TX_BYTE             ; Send word address to device
    call   RX_BYTE             ; Read data byte from device
    Movff  datai,hours
    call   BSTOP
    Return
    
Read_Data:             
;+++++++++++++  \  +++++++++++++++++++    
    call   BSTART              ; Generate Start condition
    movlw   .160            ; Load control byte for write
    movwf   datao               ; Copy to datao for output
    call   TX_BYTE             ; Send control byte to device
    movlw   5                ; Load 0x5A for word address
    movwf   datao               ; Copy to datao for output
    call   TX_BYTE
    call   BRESTART             ; Generate Start condition  
    movlw   .161                ; Load 0x00 for word address
    movwf   datao               ; Copy to datao for output
    call   TX_BYTE             ; Send word address to device
    call   RX_BYTE             ; Read data byte from device
    Movff  datai,year_day
    call   BSTOP               ; Generate Stop condition
;+++++++++++++   \ +++++++++++++++++++    
    call   BSTART              ; Generate Start condition
    movlw   .160            ; Load control byte for write
    movwf   datao               ; Copy to datao for output
    call   TX_BYTE             ; Send control byte to device
    movlw   6                ; Load 0x5A for word address
    movwf   datao               ; Copy to datao for output
    call   TX_BYTE
    call   BRESTART             ; Generate Start condition  
    movlw   .161                ; Load 0x00 for word address
    movwf   datao               ; Copy to datao for output
    call   TX_BYTE             ; Send word address to device
    call   RX_BYTE             ; Read data byte from device
    Movff  datai,weekday_month
    call   BSTOP               ; Generate Stop condition
;+++++++++++    +++++++++++++
;++++  year_day    , weekday_month     +++  
PCF:
    Movf  year_day,W
    Andlw  b'11000000'
    Movwf  year
    Swapf  year,F
    Rrncf  year,F
    Rrncf  year,W
    Addlw  .8          ;2009 
    Movwf  year
    
    
    Movf  year_day,W
    Andlw  b'00111111'
    Movwf  day
    
    Movf  weekday_month,W
    Andlw  b'11100000'
    Movwf  weekday
    Swapf  weekday,F
    Rrncf  weekday,F 
   
    Movf  weekday_month,W
    Andlw  b'00011111'
    Movwf  month
    return
    
; memory location 06 (weekdays/months)   
; 7 6 5 4 3 2 1 0
; | _ | | |____|       
;   |   |   |      
;   |   |   |      
;   |   |   |       
;   |   |    - unit months BCD
;   |    - ten months
;    - weekdays (0 to 6 binary, read as 0 if

;memory location 05 (year/date)
; 7 6 5 4 3 2 1 0
; |_| |_| |_____|       
;   |   |   |      
;   |   |   |      
;   |   |   |       
;   |   |    - unit days BCD
;   |    - ten days (0 to 3 binary)
;    - year (0 to 3 binary, read as 0 if


;*******************Byte *********************
Cloc_Write
    rcall   BSTART              ; Generate Start condition

                                ; Send control byte
    movlw   .160          ; Load control byte for write
    movwf   datao               ; Copy to datao for output
    rcall   TX_BYTE             ; Send control byte to device

                                ; Send word address high byte
    movlw   2                ; Load 0x5A for word address
    movwf   datao               ; Copy to datao for output
    rcall   TX_BYTE             ; Send high byte to device
    
    rcall   BSTART              ; Generate Start condition
                                ; Send word address low byte
    movlw   .161               ; Load 0x00 for word address
    movwf   datao               ; Copy to datao for output
    rcall   TX_BYTE             ; Send word address to device

    rcall   BRESTART            ; Generate Restart condition

                                ; Send control byte
    movlw   1            ; Load control byte for read
    movwf   datao               ; Copy to datao for output
    rcall   TX_BYTE             ; Send control byte to device
    Movff  datao,day
                                ; Read data byte
    bsf     SSPCON2,ACKDT       ; Select to send NO ACK bit
    rcall   RX_BYTE             ; Read data byte from device

    rcall   BSTOP               ; Generate Stop condition

    return

